### 准备工作

-   开发环境中下载Maven并完成环境配置。
-   开发环境中安装JDK1.8并完成环境配置（**目前项目仅支持1.8**)。


### 创建服务消费者

-   我们提供了简单的demo样例可供您快速开始了解并体验我们的平台：[服务消费者样例](https://bms-online.bj.bcebos.com/2.0/cnap-2.0.1.2_consumer-demo.zip)。

-   用户也可以在本地创建一个服务消费者工程，添加微服务注册发现依赖，开启服务注册与发现功能。并将自身通过微服务平台注册到服务注册中心：

**1.创建consumer工程**

创建一个Maven工程，命名为consumer-demo，JDK选择1.8版本。

**2.修改pom文件，添加Spring Cloud和微服务依赖**

微服务支持Spring boot 2.0.7.REALEASE及以上版本和Spring Cloud Finchley.SR2 及以上版本。
这里以Spring boot 2.0.7.REALEASE 和 Spring Cloud Finchley.SR2为例。

  微服务支持Spring boot 2.0.7.REALEASE及以上版本和Spring Cloud Finchley.SR2 及以上版本。
  这里以Spring boot 2.0.7.REALEASE 和 Spring Cloud Finchley.SR2为例。
  
      <parent>
          <groupId>org.springframework.boot</groupId>
          <artifactId>spring-boot-starter-parent</artifactId>
          <version>2.0.7.RELEASE</version>
          <relativePath/>
      </parent>
       
      <dependencyManagement>
          <dependencies>
              <dependency>
                  <groupId>org.springframework.cloud</groupId>
                  <artifactId>spring-cloud-dependencies</artifactId>
                  <version>Finchley.SR2</version>
                  <type>pom</type>
                  <scope>import</scope>
              </dependency>
          </dependencies>
      </dependencyManagement>
       
      <dependencies>
          <dependency>
              <groupId>org.springframework.boot</groupId>
              <artifactId>spring-boot-starter-web</artifactId>
          </dependency>
          <dependency>
              <groupId>com.baidubce.formula</groupId>
              <artifactId>spring-cloud-formula-cnap</artifactId>
              <version>{spring-cloud-formula.version}</version>
          </dependency>
          <dependency>
              <groupId>org.springframework.cloud</groupId>
              <artifactId>spring-cloud-starter-openfeign</artifactId>
          </dependency>
      </dependencies>

> 说明：Spring boot 2.0.7.REALEASE之前的版本存在许多已知的问题，如原有程序版本低于该版本建议升级到2.0.7.REALEASE或以上版本；

> 查看最新版本：[Spring Cloud Formula Cnap](https://mvnrepository.com/artifact/com.baidubce.formula/spring-cloud-formula-cnap)

**3.开发服务消费者启动类，开启服务发现功能**

需要使用RestTemplate、AsyncRestTemplate、FeignClient三个客户端的使用，因此需要额外添加配置：

-  添加@LoadBalance注解将RestTemplate、AsyncRestTemplate与服务发现集成
-  @EnableFeignClients注册激活FeignClient


        @SpringBootApplication
        @EnableDiscoveryClient
        @EnableFeignClients
        public class ConsumerApplication {
         
        @LoadBalanced
        @Bean
        public RestTemplate restTemplate() {
            return new RestTemplate();
        }
     
        @LoadBalanced
        @Bean
        public AsyncRestTemplate asyncRestTemplate(){
            return new AsyncRestTemplate();
        }
     
        public static void main(String[] args) {
            SpringApplication.run(ConsumerApplication.class, args);
        }
    }

**4.创建一个FeignClient接口**

FeignClient是一个将HTTP转为RPC格式调用的客户端：

    @FeignClient(name = "provider-demo")
    public interface EchoService {
        @RequestMapping(value = "/echo/{message}", method = RequestMethod.GET)
        String echo(@PathVariable("message") String message);
    }

FeignClient的名称provider-demo为服务消费者工程中配置的服务名（spring.application.name)

**5.创建一个测试Controller以演示和进行服务发现**

    @RestController
    public class TestController {
         @Autowired
         private RestTemplate restTemplate;
     
         @Autowired
         private AsyncRestTemplate asyncRestTemplate;
     
         @Autowired
         private EchoService echoService;
     
         @RequestMapping(value = "/echo-rest/{message}", method = RequestMethod.GET)
         public String rest(@PathVariable String message) {
             return restTemplate.getForObject("http://provider-demo/echo/" + message, String.class);
         }
     
         @RequestMapping(value = "/echo-async-rest/{message}", method = RequestMethod.GET)
         public String asyncRest(@PathVariable String message) throws Exception{
             ListenableFuture<ResponseEntity<String>> future = asyncRestTemplate.
             getForEntity("http://provider-demo/echo/"+message, String.class);
             return future.get().getBody();
         }
     
         @RequestMapping(value = "/echo-feign/{message}", method = RequestMethod.GET)
         public String feign(@PathVariable String message) {
             return echoService.echo(message);
         }
    }

**6.修改配置**

修改application.properties配置信息：

     spring.application.name=consumer-demo
     server.port=10002
   
使用百度CANP平台部署微服务应用, 无需配置注册中心地址和配置中心地址，可以自动获取。


### 查看路由、限流或熔断有没有被加载到

1.引入actuator包
<dependency>
   <groupId>com.baidu.formula</groupId>
   <artifactId>actuator-spring-boot-starter</artifactId>
   <version>2.0.1-SNAPSHOT</version>
</dependency>

2.查询是否加载到配置
路由：
curl http://{ip}:{port}/actuator/env | python -m json.tool |grep formula.discovery -C2
限流：
curl http://{ip}:{port}/actuator/env | python -m json.tool |grep formula.ratelimiter -C2
熔断
curl http://{ip}:{port}/actuator/env | python -m json.tool |grep formula.circuitbreaker -C2

### 本地开发与调试

> 注意：目前暂未开放本地开发调试的组件和环境，因此建议在CNAP平台进行测试。

### 服务消费者打包

打包命令
mvn clean package -U -Ponline

### CNAP平台部署

**1.制作镜像**

参考[镜像制作](https://cloud.baidu.com/doc/CNAP/s/sjwvxne56/)文档

**2.使用CNAP平台进行部署**

请参考CNAP操作手册中的服务部署部分。

**3.检查部署结果**

成功部署后将在CNAP平台 – 服务列表页中看到注册上的服务信息

![image.png](https://bce.bdstatic.com/doc/CNAP/image_aa4a7ad.png)

### 更多信息

如果想了解更多有关CANP微服务开发的内容：[百度云原生微服务应用平台开发手册](https://cloud.baidu.com/doc/CNAP/s/2jwvxndqo)