package com.baidu.bap.demo.controller;

import com.baidu.bap.demo.service.EchoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.AsyncRestTemplate;
import org.springframework.web.client.RestTemplate;

@RestController
public class EchoController {
    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private AsyncRestTemplate asyncRestTemplate;

    @Autowired
    private EchoService echoService;

    private static final Logger logger = LoggerFactory.getLogger(EchoController.class);

    @RequestMapping(value = "/echo-rest/{message}", method = RequestMethod.GET)
    public String rest(@PathVariable String message) {
        logger.info("consumer access: /echo-rest/" + message);
        return restTemplate.getForObject("http://provider-demo/echo/" + message, String.class);
    }

    @RequestMapping(value = "/echo-async-rest/{message}", method = RequestMethod.GET)
    public String asyncRest(@PathVariable String message) throws Exception {
        logger.info("consumer access: /echo-async-rest/" + message);
        ListenableFuture<ResponseEntity<String>> future = asyncRestTemplate.
                getForEntity("http://provider-demo/echo/" + message, String.class);
        return future.get().getBody();
    }

    @RequestMapping(value = "/echo-feign/{message}", method = RequestMethod.GET)
    public String feign(@PathVariable String message) {
        logger.info("consumer access: /echo-feign/" + message);
        return echoService.echo(message);
    }
}